package cn.fastfun.service.entity;


import cn.fastfun.controller.param.DeviceFormParam;
import cn.fastfun.util.ObjectUtil;

import com.bridge.entity.DateEntity;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;

import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import org.hibernate.annotations.JoinColumnOrFormula;
import org.hibernate.annotations.JoinColumnsOrFormulas;
import org.hibernate.annotations.JoinFormula;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.*;

import java.util.Date;
import java.util.Map;


@Entity
@Table(name = "app_device")
@Setter
@Getter
@NoArgsConstructor
public class AppDevice extends DateEntity {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "SN号", name = "sn", required = true)
    @Column(name = "sn")
    private String sn;

    @ApiModelProperty(value = "IMEI", name = "imei", required = true)
    @Column(name = "imei")
    private String imei;

    @ApiModelProperty(value = "批次号", name = "batchNum", required = true, example = "")
    @Column(name = "batch_num")
    private String batchNum;

    @ApiModelProperty(value = "收货地", name = "received_place", required = true, example = "")
    @Column(name = "received_place")
    private String receivedPlace;

    @ApiModelProperty(value = "状态", name = "status", required = true)
    @Column(name = "status")
    private Integer status = 0;

    // 关联
    @JsonIgnore
    @ManyToOne(targetEntity = SysDict.class)
    @JoinColumnsOrFormulas(value = {
            @JoinColumnOrFormula(column = @JoinColumn(name = "status", referencedColumnName = "name", insertable = false, updatable = false, nullable = false)),
            @JoinColumnOrFormula(formula = @JoinFormula(value = "'app_device_status'", referencedColumnName = "group_name"))
    })
    private SysDict statusDict;

    @Transient
    @JsonInclude(value = JsonInclude.Include.NON_NULL)
    public String getStatusTitle() {
        return null != statusDict ? statusDict.getValue() : "";
    }

    @ApiModelProperty(value = "操作者", name = "operator", required = true)
    @Column(name = "operator")
    private String operator;

    @ApiModelProperty(value = "发货时间", name = "deliverTime", required = true)
    @DateTimeFormat(pattern = "yyyy-MM-dd ")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @Column(name = "deliver_time")
    private Date deliverTime;


    @ApiModelProperty(value = "自检状态", name = "checkStatus", required = true)
    @Column(name = "check_status")
    private Integer checkStatus = 0;

    // 关联
    @JsonIgnore
    @ManyToOne(targetEntity = SysDict.class)
    @JoinColumnsOrFormulas(value = {
            @JoinColumnOrFormula(column = @JoinColumn(name = "check_status", referencedColumnName = "name", insertable = false, updatable = false, nullable = false)),
            @JoinColumnOrFormula(formula = @JoinFormula(value = "'app_device_check_status'", referencedColumnName = "group_name"))
    })
    private SysDict checkStatusDict;

    @Transient
    @JsonInclude(value = JsonInclude.Include.NON_NULL)
    public String getCheckStatusTitle() {
        return null != checkStatusDict ? checkStatusDict.getValue() : "";
    }


    @ApiModelProperty(value = "归属", name = "ownerId", required = true)
    @Column(name = "owner_id")
    private String ownerId;


    public AppDevice(Map<String, Object> param) {
        setBatchNum(ObjectUtil.obj2String(param.get("pathId")));
        setSn(ObjectUtil.obj2String(param.get("sn")));
        setImei(ObjectUtil.obj2String(param.get("imei")));
        setDeliverTime(ObjectUtil.obj2Date(param.get("deliverTime")));
        setReceivedPlace(ObjectUtil.obj2String(param.get("receivedPlace")));
        setStatus(1);
    }

    // 类型
    public String getType() {
        return sn.substring(0, 1);
    }

    public String getPack() {
        return sn.substring(1, 3);
    }

    // 显示名称
    @Transient
    private String typeTitle = "";

    @Transient
    private String packTitle = "";

    @Transient
    private String specTitle = "";

    @Transient
    private String expandTitle = "";


    /**
     * 从参数转为实体
     */
    public AppDevice fromFormParam(DeviceFormParam param) {
        setSn(param.getSn());
        setImei(param.getImei());
        setBatchNum(param.getBatchNum());
        setDeliverTime(param.getDeliverTime());
        setReceivedPlace(param.getReceivedPlace());
        setStatus(0);
        setCheckStatus(0);
        return this;
    }
}
